@@ -6,13 +6,17 @@ import typing
 from pathlib import Path
 
 import jsonschema
+from mercury_engine_data_structures.construct_extensions import json as json_ext
+from mercury_engine_data_structures.formats import Bmsad
 
 from open_dread_rando import elevator, lua_util
 from open_dread_rando.exefs import patch_exefs
 from open_dread_rando.logger import LOG
 from open_dread_rando.lua_editor import LuaEditor
-from open_dread_rando.patcher_editor import PatcherEditor
+from open_dread_rando.map_icons import MapIconEditor
+from open_dread_rando.patcher_editor import PatcherEditor, path_for_level
 from open_dread_rando.pickup import pickup_object_for
+from open_dread_rando.static_fixes import apply_static_fixes
 from open_dread_rando.text_patches import apply_text_patches, patch_hints, patch_text
 
 T = typing.TypeVar("T")
@@ -23,83 +27,27 @@ def _read_schema():
         return json.load(f)
 
 
-def apply_one_sided_door_fixes(editor: PatcherEditor):
-    all_scenarios = [
-        "s010_cave",
-        "s020_magma",
-        "s030_baselab",
-        "s040_aqua",
-        "s050_forest",
-        "s060_quarantine",
-        "s070_basesanc",
-        "s080_shipyard",
-    ]
-
-    for scenario_name in all_scenarios:
-        scenario = editor.get_scenario(scenario_name)
-
-        for layer_name, actor_name, actor in list(scenario.all_actors()):
-            is_door = "LIFE" in actor.pComponents and "CDoorLifeComponent" == actor.pComponents.LIFE["@type"]
-            if not is_door:
-                continue
-
-            if actor.oActorDefLink != "actordef:actors/props/doorpowerpower/charclasses/doorpowerpower.bmsad":
-                continue
-
-            life_comp = actor.pComponents.LIFE
-            left = scenario.follow_link(life_comp.wpLeftDoorShieldEntity)
-            right = scenario.follow_link(life_comp.wpRightDoorShieldEntity)
-
-            if left is None and right is None:
-                continue
-
-            elif left is None:
-                other = right
-                direction = "wpLeftDoorShieldEntity"
-
-            elif right is None:
-                other = left
-                direction = "wpRightDoorShieldEntity"
-            else:
-                continue
-
-            if "db_hdoor" in other.oActorDefLink:
-                continue
-
-            LOG.debug("%s/%s/%s: copy %s into %s", scenario_name, layer_name, actor_name, other.sName, direction)
-            mirrored = copy.deepcopy(other)
-            mirrored.sName += "_mirrored"
-            mirrored.vAng = [other.vAng[0], -other.vAng[1], other.vAng[2]]
-            scenario.actors_for_layer(layer_name)[mirrored.sName] = mirrored
-
-            link_for_actor = f"Root:pScenario:rEntitiesLayer:dctSublayers:{layer_name}:dctActors:{actor_name}"
-            mirrored_link = f"Root:pScenario:rEntitiesLayer:dctSublayers:{layer_name}:dctActors:{mirrored.sName}"
-            shield_link = f"Root:pScenario:rEntitiesLayer:dctSublayers:{layer_name}:dctActors:{other.sName}"
-
-            life_comp[direction] = mirrored_link
-
-            actor_groups = typing.cast(dict[str, list[str]],
-                                       scenario.raw.Root.pScenario.rEntitiesLayer.dctActorGroups)
-            for group_name, group_elements in actor_groups.items():
-                if link_for_actor in group_elements:
-                    if shield_link not in group_elements:
-                        # ensure the existing shield is present on both sides
-                        group_elements.append(shield_link)
-                    group_elements.append(mirrored_link)
-
-
-def apply_static_fixes(editor: PatcherEditor):
-    apply_one_sided_door_fixes(editor)
-
-
 def create_custom_init(editor: PatcherEditor, configuration: dict):
     inventory: dict[str, int] = configuration["starting_items"]
     starting_location: dict = configuration["starting_location"]
     starting_text: list[list[str]] = configuration.get("starting_text", [])
+
+    energy_per_tank = configuration.get("energy_per_tank", 100)
+    energy_per_part = configuration.get("energy_per_part", energy_per_tank / 4)
+
+    max_life = energy_per_tank - 1
     
+    # increase starting HP if starting with etanks/parts
+    if "ITEM_ENERGY_TANKS" in inventory:
+        etanks = inventory.pop("ITEM_ENERGY_TANKS")
+        max_life += etanks * energy_per_tank
+    if "ITEM_LIFE_SHARDS" in inventory and configuration.get("immediate_energy_parts"):
+        shards = inventory.pop("ITEM_LIFE_SHARDS")
+        max_life += shards * energy_per_part
+
     # Game doesn't like to start if some fields are missing, like ITEM_WEAPON_POWER_BOMB_MAX
     final_inventory = {
-        "ITEM_MAX_LIFE": 99,
+        "ITEM_MAX_LIFE": max_life,
         "ITEM_CURRENT_SPECIAL_ENERGY": 1000,
         "ITEM_MAX_SPECIAL_ENERGY": 1000,
         "ITEM_METROID_COUNT": 0,
@@ -111,8 +59,8 @@ def create_custom_init(editor: PatcherEditor, configuration: dict):
 
     def chunks(l, n):
         for i in range(0, len(l), n):
-            yield l[i:i+n]
-    
+            yield l[i:i + n]
+
     textboxes = 0
     for group in starting_text:
         boxes = chunks(group, 3)
@@ -120,14 +68,14 @@ def create_custom_init(editor: PatcherEditor, configuration: dict):
             textboxes += 1
             box_text = "|".join(box)
             patch_text(editor, f"RANDO_STARTING_TEXT_{textboxes}", box_text)
-            
 
     replacement = {
         "new_game_inventory": final_inventory,
         "starting_scenario": lua_util.wrap_string(starting_location["scenario"]),
         "starting_actor": lua_util.wrap_string(starting_location["actor"]),
         "textbox_count": textboxes,
-        "energy_per_tank": configuration.get("energy_per_tank", 100.0),
+        "energy_per_tank": energy_per_tank,
+        "energy_per_part": energy_per_part,
         "immediate_energy_parts": configuration.get("immediate_energy_parts", False),
     }
 
@@ -137,11 +85,12 @@ def create_custom_init(editor: PatcherEditor, configuration: dict):
 def patch_pickups(editor: PatcherEditor, lua_scripts: LuaEditor, pickups_config: list[dict], configuration: dict):
     # add to the TOC
     editor.add_new_asset("actors/items/randomizer_powerup/scripts/randomizer_powerup.lc", b'', [])
+    map_icon_editor = MapIconEditor(editor)
 
     for i, pickup in enumerate(pickups_config):
         LOG.debug("Writing pickup %d: %s", i, pickup["resources"][0]["item_id"])
         try:
-            pickup_object_for(lua_scripts, pickup, i, configuration).patch(editor)
+            pickup_object_for(lua_scripts, pickup, i, configuration, map_icon_editor).patch(editor)
         except NotImplementedError as e:
             LOG.warning(e)
 
@@ -188,7 +137,7 @@ def patch(input_path: Path, output_path: Path, configuration: dict):
     # Hints
     if "hints" in configuration:
         patch_hints(editor, configuration["hints"])
-    
+
     # Text patches
     if "text_patches" in configuration:
         apply_text_patches(editor, configuration["text_patches"])
